<?php

namespace Drupal\fasttoggle\Controller;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Route controller for Fasttoggle.
 */
class FasttoggleController extends ControllerBase {

  /**
   * Toggle a setting.
   *
   * @param Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   */
  public function toggle(Request $request) {
    $parts = explode('/', $request->getPathInfo());
    $type = $parts[2];
    $id = $parts[3];
    $action = $parts[4];
    $config = $this->config('fasttoggle.settings');
    $label_style = $config->get('label_style');

    if (count($parts) != 4 && count($parts) != 5) {
      $response = new Response();
      $response->setContent('Incorrect number of parameters');
      $response->setStatusCode(400);
      return $response;
    }

    switch ($type) {
      case 'node':
        $storage_handler = $this->entityTypeManager()->getStorage('node');
        $node = $storage_handler->load($id);

        if (!$node) {
          $response = new Response();
          $response->setContent('Invalid entity ID.');
          $response->setStatusCode(400);
          return $response;
        }

        switch ($action) {
          case 'status':
            $status = $node->isPublished();
            switch ($label_style) {
              case 0:
                !$node->isPublished() ? $link_title = t('Published') : $link_title = t('Not published');
                break;

              case 1:
                !$node->isPublished() ? $link_title = t('Unpublish') : $link_title = t('Publish');
                break;
            }
            $node->setPublished(!$status);
            $selector = '.fasttoggle-node-status';
            $data = '<a href="/fasttoggle/node/' . $id . '/status" class="use-ajax fasttoggle-node-status">' . $link_title . '</a>';
            break;

          case 'promote':
            $promote = $node->isPromoted();
            switch ($label_style) {
              case 0:
                !$node->isPromoted() ? $link_title = t('Promoted') : $link_title = t('Not promoted');
                break;

              case 1:
                !$node->isPromoted() ? $link_title = t('Demote') : $link_title = t('Promote');
                break;
            }
            $node->setPromoted(!$promote);
            $selector = '.fasttoggle-node-promote';
            $data = '<a href="/fasttoggle/node/' . $id . '/promote" class="use-ajax fasttoggle-node-promote">' . $link_title . '</a>';
            break;
        }

        $node->save();

        $response = new AjaxResponse();
        $response->addCommand(new ReplaceCommand($selector, $data, $settings = NULL));
        return $response;

      default:
        $response = new Response();
        $response->setContent('Invalid entity type.');
        $response->setStatusCode(400);
        return $response;
    }
  }

}
